@@ -48,16 +48,27 @@
 #define PGrnMatchInStrategyV2DeprecatedNumber	26
 /* operator &?> (multiple conditions of query in Groonga). Deprecated since 1.2.1. */
 #define PGrnQueryInStrategyV2DeprecatedNumber	27
-#define PGrnQueryStrategyV2Number	28	/* operator &?  (query in Groonga) */
-#define PGrnSimilarStrategyV2Number	29	/* operator &~? (similar search) */
-#define PGrnQueryInStrategyV2Number	30	/* operator &?| (multiple conditions of query in Groonga) */
+#define PGrnQueryStrategyV2Number	28	/* operator &@~  (query in Groonga) */
+#define PGrnSimilarStrategyV2Number	29	/* operator &@* (similar search) */
+#define PGrnQueryInStrategyV2Number	30	/* operator &@~| (multiple conditions of query in Groonga) */
+/* operator &@ with pgroonga_match_condition */
+#define PGrnMatchConditionStrategyV2Number	31
+/* operator &@~ with pgroonga_match_condition */
+#define PGrnQueryConditionStrategyV2Number	32
+/* operator &@ with pgroonga_match_condition_with_scorers */
+#define PGrnMatchConditionWithScorersStrategyV2Number	33
+/* operator &@~ with pgroonga_match_condition_with_scorers */
+#define PGrnQueryConditionWithScorersStrategyV2Number	34
 
-#define PGRN_N_STRATEGIES PGrnQueryInStrategyV2Number
+#define PGRN_N_STRATEGIES PGrnQueryConditionWithScorersStrategyV2Number
 
 /* file and table names */
 #define PGrnLogPathDefault				"pgroonga.log"
 #define PGrnQueryLogPathDefault			"none"
 #define PGrnDatabaseBasename			"pgrn"
+#define PGrnBuildingSourcesTableNamePrefix	"BuildingSources"
+#define PGrnBuildingSourcesTableNamePrefixLength	(sizeof(PGrnBuildingSourcesTableNamePrefix) - 1)
+#define PGrnBuildingSourcesTableNameFormat	PGrnBuildingSourcesTableNamePrefix "%u"
 #define PGrnSourcesTableNamePrefix		"Sources"
 #define PGrnSourcesTableNamePrefixLength	(sizeof(PGrnSourcesTableNamePrefix) - 1)
 #define PGrnSourcesTableNameFormat		PGrnSourcesTableNamePrefix "%u"
@@ -69,9 +80,17 @@
 #define PGrnJSONValuesTableNameFormat	PGrnJSONValuesTableNamePrefix "%u_%u"
 #define PGrnJSONTypesTableNamePrefix	"JSONTypes"
 #define PGrnJSONTypesTableNameFormat	PGrnJSONTypesTableNamePrefix "%u_%u"
-#define PGrnJSONValueLexiconNameFormat	"JSONValueLexicon%s%u_%u"
-#define PGrnLexiconNameFormat			"Lexicon%u_%u"
+#define PGrnJSONValueLexiconNamePrefix	"JSONValueLexicon"
+#define PGrnJSONValueLexiconNameFormat	PGrnJSONValueLexiconNamePrefix "%s%u_%u"
+#define PGrnLexiconNamePrefix			"Lexicon"
+#define PGrnLexiconNameFormat			PGrnLexiconNamePrefix "%u_%u"
 #define PGrnIndexColumnName				"index"
 
+#define PGRN_EXPR_QUERY_PARSE_FLAGS				\
+	(GRN_EXPR_SYNTAX_QUERY |					\
+	 GRN_EXPR_ALLOW_LEADING_NOT |				\
+	 GRN_EXPR_QUERY_NO_SYNTAX_ERROR)
+
 extern bool PGrnGroongaInitialized;
+void PGrnEnsureDatabase(void);
 bool PGrnIndexIsPGroonga(Relation index);
